

import ij.IJ;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ini.trakem2.display.*;
import ini.trakem2.Project;
import ini.trakem2.ControlWindow;
import ini.trakem2.persistence.Loader;
import ini.trakem2.persistence.DBLoader;
import java.util.*;


/** Calls remake() on all Patch snapshots, which updates the database copy as well.
 * 
 *  General Public License applies. USE AT YOU OWN RISK
 *
 * */
public class T2_Remake_All_Snapshots implements PlugIn {

	public void run(String arg) {
		try {
			Project project = null;
			ArrayList al_p = new ArrayList();
			ArrayList al_t = new ArrayList();
			for (Iterator it = ControlWindow.getProjects().iterator(); it.hasNext(); ) {
				Project p = (Project)it.next();
				al_p.add(p);
				al_t.add(p.getTitle());
			}
			if (0 == al_p.size()) return;
			if (1 == al_p.size()) project = (Project)al_p.get(0);
			else {
				String[] titles = new String[al_t.size()];
				al_t.toArray(titles);
				final GenericDialog gd = new GenericDialog("Choose project");
				gd.addChoice("project: ", titles, titles[0]);
				gd.showDialog();
				if (gd.wasCanceled()) return;
				project = (Project)al_p.get(gd.getNextChoiceIndex());
			}
			if (!(project.getLoader() instanceof DBLoader)) {
				IJ.showMessage("Selected project is not a database project. No need to update snapshots.");
				return;
			}
			final LayerSet layer_set = project.getRootLayerSet();
			final Loader loader = project.getLoader();
			for (Iterator it = layer_set.getLayers().iterator(); it.hasNext(); ) {
				final Layer layer = (Layer)it.next();

				// disable autocommit
				loader.startLargeUpdate();
				loader.setMassiveMode(true);
				System.out.println("processing " + layer);
				for (Iterator pit = layer.getDisplayables().iterator(); pit.hasNext(); ) {
					Object ob = pit.next();
					if (ob instanceof Patch) {
						try {
							((Patch)ob).getSnapshot().remake();
						} catch (Exception e) {
							System.out.println("remake failed for patch id=" + ((Patch)ob).getId());
						}
					}
				}
				// commit for the layer
				loader.commitLargeUpdate();
				// sleep between layers
				try {
					Thread.sleep(1000);
				} catch (Exception e) {}
			}
		} catch (Exception ae) {
			ae.printStackTrace();
		}
	}
}
